@@ -1,34 +1,16 @@
 #!/usr/bin/env python3
 # coding: utf-8
 
-import importlib
+import abc
 import logging
 import os
 import re
-import sys
 import weakref
 
-import json5
-
 from volkanic import utils
-from volkanic.compat import cached_property
+from volkanic.compat import cached_property, abstract_property
 
-logger = logging.getLogger(__name__)
-
-
-class Singleton(object):
-    registered_instances = {}
-
-    def __new__(cls, *args, **kwargs):
-        try:
-            return cls.registered_instances[cls]
-        except KeyError:
-            obj = super(Singleton, cls).__new__(cls, *args, **kwargs)
-            return cls.registered_instances.setdefault(cls, obj)
-
-
-class WeakSingleton(object):
-    registered_instances = weakref.WeakValueDictionary()
+_logger = logging.getLogger(__name__)
 
 
 class SingletonMeta(type):
@@ -42,15 +24,27 @@ class SingletonMeta(type):
             return cls.registered_instances.setdefault(cls, obj)
 
 
+class WeakSingletonMeta(SingletonMeta):
+    registered_instances = weakref.WeakValueDictionary()
+
+
+class Singleton(metaclass=SingletonMeta):
+    pass
+
+
+class WeakSingleton(metaclass=WeakSingletonMeta):
+    pass
+
+
 class _GIMeta(SingletonMeta):
     def __new__(mcs, name, bases, attrs):
         pn = attrs.get('package_name')
+        if pn is None:
+            msg = '{}.package_name is missing'.format(name)
+            raise ValueError(msg)
         if not isinstance(pn, str):
             msg = '{}.package_name is of wrong type'.format(name)
             raise TypeError(msg)
-        if not pn:
-            msg = '{}.package_name is missing'.format(name)
-            raise ValueError(msg)
         if not re.match(r'\w[\w.]*\w', pn):
             msg = 'invalid {}.package_name: "{}"'.format(name, pn)
             raise ValueError(msg)
@@ -58,13 +52,13 @@ class _GIMeta(SingletonMeta):
 
 
 class _PackageNameDerivant:
-    __slots__ = ['_replacement']
+    __slots__ = ['_sep']
 
-    def __init__(self, replacement):
-        self._replacement = replacement
+    def __init__(self, sep):
+        self._sep = sep
 
     def __get__(self, _, owner):
-        return owner.package_name.replace('.', self._replacement)
+        return owner.package_name.replace('.', self._sep)
 
 
 class GlobalInterface(metaclass=_GIMeta):
@@ -79,11 +73,19 @@ class GlobalInterface(metaclass=_GIMeta):
     # dot '.' in package_name replaced by underscore '_'
     identifier = _PackageNameDerivant('_')
 
-    # for project dir locating (under_project_dir())
-    project_source_depth = 0
+    _options = {
+        # for project dir locating (under_project_dir())
+        'project_source_depth': 0,
+        # for config file locating (_get_conf_paths())
+        'confpath_filename': 'config.json5',
+        # for config file locating (_get_conf_paths())
+        'confpath_dirname_sep': '-',
+    }
 
     # default config and log format
-    default_config = {}
+    default_config = {
+        '_jinja2_env': {},
+    }
     default_logfmt = \
         '%(asctime)s %(levelname)s [%(process)s,%(thread)s] %(name)s %(message)s'
 
@@ -91,6 +93,19 @@ class GlobalInterface(metaclass=_GIMeta):
     def _fmt_envvar_name(cls, name):
         return '{}_{}'.format(cls.identifier, name).upper()
 
+    @classmethod
+    def _get_option(cls, key: str):
+        for c in cls.mro():
+            options = c.__dict__.get('_options')
+            try:
+                return options[key]
+            except (KeyError, TypeError):
+                pass
+
+    @classmethod
+    def _fmt_name(cls, sep='-'):
+        return cls.package_name.replace('.', sep)
+
     @classmethod
     def _get_conf_paths(cls):
         """
@@ -98,15 +113,18 @@ class GlobalInterface(metaclass=_GIMeta):
         Override this method in your subclasses for your specific project.
         """
         envvar_name = cls._fmt_envvar_name('confpath')
-        pn = cls.project_name
+        fn = cls._get_option('confpath_filename')
+        pn = cls._fmt_name(cls._get_option('confpath_dirname_sep'))
         return [
             os.environ.get(envvar_name),
-            cls.under_project_dir('config.json5'),
-            cls.under_home_dir('.{}/config.json5'.format(pn)),
-            '/etc/{}/config.json5'.format(pn),
-            '/{}/config.json5'.format(pn),
+            cls.under_project_dir(fn),
+            utils.under_home_dir('.{}/{}'.format(pn, fn)),
+            '/etc/{}/{}'.format(pn, fn),
+            '/{}/{}'.format(pn, fn),
         ]
 
+    # _get_conf_search_paths is deprecated
+    # _get_conf_search_paths will be remove at ver 0.5.0
     _get_conf_search_paths = _get_conf_paths
 
     @classmethod
@@ -114,6 +132,8 @@ class GlobalInterface(metaclass=_GIMeta):
         """
         Returns: (str) absolute path to config file
         """
+        # _get_conf_search_paths is deprecated
+        # _get_conf_search_paths will be remove at ver 0.5.0
         func = getattr(cls, '_get_conf_search_paths', None)
         if func is None:
             func = cls._get_conf_paths
@@ -126,7 +146,11 @@ class GlobalInterface(metaclass=_GIMeta):
 
     @staticmethod
     def _parse_conf(path: str):
-        return json5.load(open(path))
+        return utils.load_json5_file(path)
+
+    @staticmethod
+    def _check_conf(config: dict):
+        return config
 
     @cached_property
     def conf(self) -> dict:
@@ -134,11 +158,12 @@ class GlobalInterface(metaclass=_GIMeta):
         cn = self.__class__.__name__
         if path:
             config = self._parse_conf(path)
-            print('{}.conf, path'.format(cn), path, file=sys.stderr)
+            utils.printerr('{}.conf, path'.format(cn), path)
         else:
             config = {}
-            print('{}.conf, hard-coded'.format(cn), file=sys.stderr)
-        return utils.merge_dicts(self.default_config, config)
+            utils.printerr('{}.conf, hard-coded'.format(cn))
+        config = utils.merge_dicts(self.default_config, config)
+        return self._check_conf(config)
 
     @staticmethod
     def under_home_dir(*paths):
@@ -153,18 +178,21 @@ class GlobalInterface(metaclass=_GIMeta):
         pkg_dir = cls.under_package_dir()
         if re.search(r'[/\\](site|dist)-packages[/\\]', pkg_dir):
             return
-        n = cls.project_source_depth
+        n = cls._get_option('project_source_depth')
         n += len(cls.package_name.split('.'))
         paths = ['..'] * n + list(paths)
         return utils.abs_path_join(pkg_dir, *paths)
 
+    # deprecated
+    # this method will be removed at ver 0.5.0
     @cached_property
     def jinja2_env(self):
         # noinspection PyPackageRequirements
         from jinja2 import Environment, PackageLoader, select_autoescape
         return Environment(
             loader=PackageLoader(self.package_name, 'templates'),
-            autoescape=select_autoescape(['html', 'xml'])
+            autoescape=select_autoescape(['html', 'xml']),
+            **self.conf.get('_jinja2_env', {})
         )
 
     @classmethod
@@ -174,3 +202,44 @@ class GlobalInterface(metaclass=_GIMeta):
             level = os.environ.get(envvar_name, 'DEBUG')
         fmt = fmt or cls.default_logfmt
         logging.basicConfig(level=level, format=fmt)
+
+
+# deprecated
+# this class will be removed at ver 0.4.0
+class GIMixinDirs:
+    @abstract_property
+    def conf(self) -> dict:
+        return NotImplemented
+
+    @abc.abstractmethod
+    def under_project_dir(self, *paths) -> str:
+        return NotImplemented
+
+    def _under_data_dir(self, conf_key, *paths, mkdirs=False) -> str:
+        dirpath = self.conf[conf_key]
+        if not mkdirs:
+            return utils.abs_path_join(dirpath, *paths)
+        return utils.abs_path_join_and_mkdirs(dirpath, *paths)
+
+    def under_data_dir(self, *paths, mkdirs=False) -> str:
+        return self._under_data_dir('data_dir', *paths, mkdirs=mkdirs)
+
+    def _under_resources_dir(self, conf_key, default, *paths) -> str:
+        dirpath = self.conf.get(conf_key)
+        if not dirpath:
+            dirpath = self.under_project_dir(default)
+        if not dirpath or not os.path.isdir(dirpath):
+            raise NotADirectoryError(dirpath)
+        return utils.abs_path_join(dirpath, *paths)
+
+    def under_resources_dir(self, *paths) -> str:
+        f = self._under_resources_dir
+        return f('resources_dir', 'resources', *paths)
+
+    def under_temp_dir(self, ext=''):
+        name = os.urandom(17).hex() + ext
+        return self.under_data_dir('tmp', name, mkdirs=True)
+
+    # both will be removed
+    get_temp_path = under_temp_dir
+    under_temp_path = under_temp_dir
