@@ -1,6 +1,7 @@
 #!/usr/bin/env python3
 # coding: utf-8
 
+import functools
 import importlib
 import os
 import sys
@@ -20,17 +21,28 @@ def attr_setdefault(obj, attrname, value):
         return value
 
 
+# deprecated names
 query_attr = attr_query
+subattr = query_attr
 
 
 def merge_dicts(*dicts):
+    # a list of dicts is acceptable
+    if len(dicts) == 1 and isinstance(dicts[0], list):
+        dicts = dicts[0]
     retdic = {}
     for dic in dicts:
         retdic.update(dic)
     return retdic
 
 
-subattr = query_attr
+def select_from_dict(dict_: dict, fields: list, pop=False):
+    sub_dict = {}
+    method = dict_.pop if pop else dict_.get
+    for key in fields:
+        if key in dict_:
+            sub_dict[key] = method(key, None)
+    return sub_dict
 
 
 def load_symbol(symbolpath):
@@ -158,4 +170,31 @@ def indented_json_print(obj, **kwargs):
     for key in print_keywords:
         if key in kwargs:
             print_kwargs[key] = kwargs.pop(key)
-    print(indented_json_dumps(obj, **kwargs), **print_kwargs)
+    dumps = kwargs.pop('dumps', indented_json_dumps)
+    print(dumps(obj, **kwargs), **print_kwargs)
+
+
+def load_json5_file(path: str):
+    if path.endswith('.json'):
+        import json
+        return json.loads(open(path).read())
+    import json5
+    return json5.load(open(path))
+
+
+def ignore_arguments(func):
+    """Discard arguments and call a argument-less function"""
+
+    @functools.wraps(func)
+    def _func(*_args, **_kwargs):
+        return func()
+
+    return _func
+
+
+discard_arguments = ignore_arguments
+
+
+def printerr(*args, **kwargs):
+    kwargs.setdefault('file', sys.stderr)
+    print(*args, **kwargs)
